Package: heatmaply
Type: Package
Title: Interactive Cluster Heat Maps Using 'plotly' and 'ggplot2'
Version: 1.6.0
Date: 2025-07-12
Authors@R: c(
    person("Tal", "Galili", role = c("aut", "cre", "cph"), email = "tal.galili@gmail.com", comment = "https://www.r-statistics.com"),
    person("Alan", "O'Callaghan", comment = "https://github.com/Alanocallaghan",role = "aut"),
    person("Jonathan", "Sidi", email = "yonis@metrumrg.com", comment = "https://github.com/yonicd",role = "ctb"),
    person("Jaehyun", "Joo", comment = "https://github.com/jaehyunjoo",role = "ctb"),
    person("Yoav", "Benjamini", email = "ybenja@tau.ac.il",role = "ths"),
    person("Mathew", "Simenc", role = "ctb", comment = "https://gitlab.com/mcsimenc, https://github.com/mcsimenc"))
Description: Create interactive cluster 'heatmaps' that can be saved as a stand-
    alone HTML file, embedded in 'R Markdown' documents or in a 'Shiny' app, and
    available in the 'RStudio' viewer pane. Hover the mouse pointer over a cell to
    show details or drag a rectangle to zoom. A 'heatmap' is a popular graphical
    method for visualizing high-dimensional data, in which a table of numbers
    are encoded as a grid of colored cells. The rows and columns of the matrix
    are ordered to highlight patterns and are often accompanied by 'dendrograms'.
    'Heatmaps' are used in many fields for visualizing observations, correlations,
    missing values patterns, and more. Interactive 'heatmaps' allow the inspection
    of specific value by hovering the mouse over a cell, as well as zooming into
    a region of the 'heatmap' by dragging a rectangle around the relevant area.
    This work is based on the 'ggplot2' and 'plotly.js' engine. It produces
    similar 'heatmaps' to 'heatmap.2' with the advantage of speed
    ('plotly.js' is able to handle larger size matrix), the ability to zoom from
    the 'dendrogram' panes, and the placing of factor variables in the sides of the
    'heatmap'.
Depends: R (>= 3.0.0), plotly (>= 4.7.1), viridis
Imports: ggplot2 (>= 2.2.0), dendextend (>= 1.12.0), magrittr (>=
        1.0.1), reshape2, scales, seriation, utils, stats, grDevices,
        methods, colorspace, RColorBrewer, htmlwidgets, webshot,
        assertthat, egg
Suggests: knitr, covr, gplots, tidyselect, rmarkdown, testthat
VignetteBuilder: knitr
License: GPL-2 | GPL-3
URL: https://talgalili.github.io/heatmaply/,
        https://cran.r-project.org/package=heatmaply,
        https://github.com/talgalili/heatmaply/,
        https://www.r-statistics.com/tag/heatmaply/
BugReports: https://github.com/talgalili/heatmaply/issues
RoxygenNote: 7.3.2
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2025-07-12 16:09:42 UTC; talgalili
Author: Tal Galili [aut, cre, cph] (https://www.r-statistics.com),
  Alan O'Callaghan [aut] (https://github.com/Alanocallaghan),
  Jonathan Sidi [ctb] (https://github.com/yonicd),
  Jaehyun Joo [ctb] (https://github.com/jaehyunjoo),
  Yoav Benjamini [ths],
  Mathew Simenc [ctb] (https://gitlab.com/mcsimenc,
    https://github.com/mcsimenc)
Maintainer: Tal Galili <tal.galili@gmail.com>
Repository: CRAN
Date/Publication: 2025-07-12 21:20:02 UTC
Built: R 4.6.0; ; 2026-04-30 07:40:47 UTC; unix
