# Generated by roxygen2: do not edit by hand

S3method(heatmaply,default)
S3method(heatmaply,heatmapr)
S3method(normalize,data.frame)
S3method(normalize,default)
S3method(normalize,matrix)
S3method(percentize,data.frame)
S3method(percentize,default)
S3method(percentize,matrix)
export(Blues)
export(BrBG)
export(BuGn)
export(BuPu)
export(GnBu)
export(Greens)
export(Greys)
export(OrRd)
export(Oranges)
export(PRGn)
export(PiYG)
export(PuBu)
export(PuBuGn)
export(PuOr)
export(PuRd)
export(Purples)
export(RdBu)
export(RdGy)
export(RdPu)
export(RdYlBu)
export(RdYlGn)
export(Reds)
export(Spectral)
export(YlGn)
export(YlGnBu)
export(YlOrBr)
export(YlOrRd)
export(cool_warm)
export(ggheatmap)
export(heatmaply)
export(heatmaply_cor)
export(heatmaply_na)
export(heatmapr)
export(is.heatmapr)
export(is.na10)
export(is.plotly)
export(normalize)
export(percentize)
import(RColorBrewer)
import(dendextend)
import(ggplot2)
import(magrittr)
import(webshot)
importFrom(assertthat,assert_that)
importFrom(dendextend,as.ggdend)
importFrom(dendextend,color_branches)
importFrom(dendextend,find_k)
importFrom(dendextend,is.dendrogram)
importFrom(dendextend,is.hclust)
importFrom(dendextend,rotate)
importFrom(dendextend,seriate_dendrogram)
importFrom(dendextend,set)
importFrom(grDevices,col2rgb)
importFrom(grDevices,dev.new)
importFrom(grDevices,rgb)
importFrom(htmlwidgets,saveWidget)
importFrom(methods,as)
importFrom(plotly,add_segments)
importFrom(plotly,add_trace)
importFrom(plotly,config)
importFrom(plotly,ggplotly)
importFrom(plotly,hide_colorbar)
importFrom(plotly,layout)
importFrom(plotly,plot_ly)
importFrom(plotly,plotly)
importFrom(plotly,plotly_empty)
importFrom(plotly,style)
importFrom(plotly,subplot)
importFrom(scales,col_factor)
importFrom(scales,col_numeric)
importFrom(seriation,get_order)
importFrom(seriation,seriate)
importFrom(stats,as.dendrogram)
importFrom(stats,as.dist)
importFrom(stats,cor)
importFrom(stats,dendrapply)
importFrom(stats,dist)
importFrom(stats,ecdf)
importFrom(stats,hclust)
importFrom(stats,is.leaf)
importFrom(stats,na.omit)
importFrom(stats,order.dendrogram)
importFrom(stats,reorder)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(viridis,viridis)
